'''
filters a given GIIRA GTF file for all genes with sufficient support
Copyright (c) 2013,
Franziska Zickmann, 
ZickmannF@rki.de, Robert Koch-Institute, Berlin, Germany
Distributed under the GNU Lesser General Public License, version 3.0
'''

import sys
import numpy

if len(sys.argv) <= 4:
	print "Script to filter a given GIIRA GTF file for all genes with sufficient support."
	print "Usage: python filterGenes.py [Path_TO_GTF] [PATH_OUTFILE] [WANT_AMBI_FILTER: (y/n)] [WANT_COVERAGE_FILTER: (y/n)] [WANT_UNIQUE_COVERAGE_FILTER: (y/n)]"
	print "AMBI-FILTER = tag \"alsoUniqueSupport\" in GIIRA output"
	print "COVERAGE-FILTER = tag \"coverageSupport\" in GIIRA output"
	print "UNIQUE_COVERAGE-FILTER = tag \"hasEnoughUniques\" in GIIRA output"
	print "Example: python filterGenes.py myGenes.gtf myGenes_filtered.gtf y y n"
	print "This applies a filtering for genes only supported by ambiguous reads or lacking sufficient overall coverage."
	sys.exit(1)
if "-h" in sys.argv[1]:
	print "Script to filter a given GIIRA GTF file for all genes with sufficient support."
	print "Usage: python filterGenes.py [Path_TO_GTF] [PATH_OUTFILE] [WANT_AMBI_FILTER: (y/n)] [WANT_COVERAGE_FILTER: (y/n)] [WANT_UNIQUE_COVERAGE_FILTER: (y/n)]"
	print "AMBI-FILTER = tag \"alsoUniqueSupport\" in GIIRA output"
	print "COVERAGE-FILTER = tag \"coverageSupport\" in GIIRA output"
	print "UNIQUE_COVERAGE-FILTER = tag \"hasEnoughUniques\" in GIIRA output"
	print "Example: python filterGenes.py myGenes.gtf myGenes_filtered.gtf y y y"
	print "This applies a filtering for genes only supported by ambiguous reads or lacking sufficient overall coverage."
	sys.exit(1)

pathToGTF = sys.argv[1]
pathOutFile = sys.argv[2]
wantMultiFilter = sys.argv[3]
wantCovFilter = sys.argv[4]
wantUniqueCovfilter = sys.argv[5]

infile = open(pathToGTF, 'r')
outfile = open(pathOutFile, 'w')

for line in infile:
	arr = line.rstrip().split("\t")
	arrTag = arr[8].split(";")
	wantLine = True
	
	if "y" in wantMultiFilter:
		if ": n" in arrTag[len(arrTag)-4]:
			wantLine = False
	if "y" in wantCovFilter:	
		if ": n" in arrTag[len(arrTag)-3]:
			wantLine = False
	if "y" in wantUniqueCovfilter:	
		if ": n" in arrTag[len(arrTag)-2]:
			wantLine = False
	
	if wantLine:
		outfile.write(line);


# -*- coding: utf-8 -*-
"""
Copyright (c) 2013,
Franziska Zickmann, 
ZickmannF@rki.de, Robert Koch-Institute, Berlin, Germany
Distributed under the GNU Lesser General Public License, version 3.0
"""
#!/usr/bin/env python
import os
import optparse

parser = optparse.OptionParser()

args = parser.parse_args()

fastqFiles = args[1][0]
output = args[1][1]

fileArr = fastqFiles.split("&&")

fileNames = ""

for file in fileArr:
	fileNames = "%s %s" % (fileNames,file)

syscall = "cat %s > %s" %(fileNames,output)
os.system(syscall)



"""
Copyright (c) 2013,
Franziska Zickmann, 
ZickmannF@rki.de, Robert Koch-Institute, Berlin, Germany
Distributed under the GNU Lesser General Public License, version 3.0
"""

import pysam
import matplotlib.pyplot as plt
import numpy as np
from math import *
import scipy.stats as stats
import sys

# calculate coverage without x-coverage bases
def covWOzero(cov,x):
	cov2 = np.array([])
	count = 0
	for i in cov:
		if i!= x:
			count += 1
			cov2.resize(count)
			cov2[count-1]=i
	return cov2
	
# only computes the maxCov without the need to refine the coverage map
def computeMaxCov(cov,maxCov,it):
	if it > 100:
		return cov
	# first remove all maxCov entries
	while (np.max(cov)) == maxCov:
		cov = np.delete(cov,(len(cov)-1))
	
	mean = np.mean(cov)
	median = np.median(cov)	
	if (median * 10.0) < mean :
		maxCov = np.max(cov)
		cov = computeMaxCov(cov,maxCov,(it+1))
		maxCov = np.max(cov)
		
	return cov
	
nameIn = sys.argv[1]       # name and path sam file
nameOut = sys.argv[2]

""" extract a genome coverage profile from a sam file. """
sf = pysam.Samfile(nameIn,'r')
	
cov = np.zeros((sum(sf.lengths),))
start_pos = np.cumsum(sf.lengths)-sf.lengths[0]
read_length = 0
num_reads = 0
for read in sf:
	if not read.is_unmapped:
		r_start = start_pos[read.tid] + read.pos # start position 
		r_end = start_pos[read.tid] + read.pos + read.qlen # end 
		cov[r_start:r_end] += 1 
		num_reads += 1
		read_length += r_end-r_start
				

#print "length including zero: %s" %(len(cov))

# calculate coverage

covWOZRef = covWOzero(cov,0)

#print "length without zero: %s" %(len(covWOZRef))

mean_cov_wozRef = np.mean(covWOZRef)
percentileQuart_cov_wozRef = np.percentile(covWOZRef,25)
median_cov_wozRef = np.median(covWOZRef)
percentileUpQuart_cov_wozRef = np.percentile(covWOZRef,75)

print "average: %s" %(mean_cov_wozRef)
print "median: %s" %(median_cov_wozRef)
print "25-quart: %s" %(percentileQuart_cov_wozRef)
print "75-quart: %s" %(percentileUpQuart_cov_wozRef)

maxCov = -1
if (median_cov_wozRef * 10.0) < mean_cov_wozRef:
	sys.setrecursionlimit(100000)
	covSorted = np.sort(covWOZRef)
	print "finished sorting!"
	maxCov = np.max(covSorted)
	itNum = 1
	while True:
		covSorted = computeMaxCov(covSorted,maxCov,1)
		maxCov = np.max(covSorted)
		mean = np.mean(covSorted)
		median = np.median(covSorted)
		print "iter %s max: %s, median: %s, average: %s" %(itNum,maxCov,median,mean) 
		itNum = itNum + 1
		if (median * 10.0) >= mean:
			break
	
	
print "maximum threshold: %s" %(maxCov)

outfile = open(nameOut,'w')
outfile.write(str(min(percentileQuart_cov_wozRef,(mean_cov_wozRef/5.0))) + "\n");
outfile.write(str(maxCov) + "\n")
outfile.write("25-qua: " + str(percentileQuart_cov_wozRef) + "\n")
outfile.write("median: " + str(median_cov_wozRef) + "\n")
outfile.write("average: " + str(mean_cov_wozRef) + "\n")
outfile.close()





# -*- coding: utf-8 -*-
"""
Created on Feb 03 2012
helper script to call bwa index 
Copyright (c) 2013,
Franziska Zickmann, 
ZickmannF@rki.de, Robert Koch-Institute, Berlin, Germany
Distributed under the GNU Lesser General Public License, version 3.0
"""
#!/usr/bin/env python
import os
import optparse
import subprocess

parser = optparse.OptionParser()

args = parser.parse_args()

orfFile = args[1][0]
print orfFile
para = args[1][1]
print para
fnull = open(os.devnull, 'w')

syscall = "bwa index -a %s %s" %(para, orfFile)
print syscall
result = subprocess.call(syscall, shell = True, stdout = fnull, stderr = fnull)
fnull.close()



# -*- coding: utf-8 -*-
"""
Copyright (c) 2013,
Franziska Zickmann, 
ZickmannF@rki.de, Robert Koch-Institute, Berlin, Germany
Distributed under the GNU Lesser General Public License, version 3.0
"""
#!/usr/bin/env python
import os
import sys

pathOut = sys.argv[1]

syscall1 = "samtools view -b -S %sresultsRun/reassignedReads.sam > %sresultsRun/reassignedReads.bam" %(pathOut,pathOut)
print syscall1
os.system(syscall1)

syscall2 = "samtools sort -n %sresultsRun/reassignedReads.bam %sresultsRun/reassignedReads_sorted" %(pathOut,pathOut)
print syscall2
os.system(syscall2)

syscall3 = "samtools view -h -o %sresultsRun/reassignedReads_sorted.sam %sresultsRun/reassignedReads_sorted.bam" %(pathOut,pathOut)
print syscall3
os.system(syscall3)


# -*- coding: utf-8 -*-
"""
Created on Oct 31 16:29:55 2011
helper script to call bwa samse
Copyright (c) 2013,
Franziska Zickmann, 
ZickmannF@rki.de, Robert Koch-Institute, Berlin, Germany
Distributed under the GNU Lesser General Public License, version 3.0
"""
#!/usr/bin/env python
import os
import optparse
import subprocess

parser = optparse.OptionParser()

args = parser.parse_args()

paras = args[1][0]
orfFile = args[1][1]
rnaFile = args[1][2]
outFile = args[1][3]

callPara = " "
fnull = open(os.devnull, 'w')

if paras == 0:
        syscall = "bwa bwasw %s %s > %s" %(orfFile,rnaFile,outFile)
        #os.system(syscall)
        result = subprocess.call(syscall, shell = True, stdout = fnull, stderr = fnull)
        fnull.close()
else:
        stringAr = paras[1:(len(paras)-1)].split("_")
        for data in stringAr:
                callPara=callPara+data+" "
        syscall = "bwa bwasw%s %s %s> %s" %(callPara,orfFile,rnaFile,outFile)
        #os.system(syscall)
        print syscall
        result = subprocess.call(syscall, shell = True, stdout = fnull, stderr = fnull)
        fnull.close()





'''
Created on 4 November 2013

calls the glpsol lp solver to assign the rnas

@author: zickmannf
'''

import sys
import numpy
import os

inputFile = sys.argv[1]
outputFile = sys.argv[2]

syscall = "glpsol --lp %s --output %s" %(inputFile,outputFile)
os.system(syscall)
